@@ -14,7 +14,6 @@
 #' @param numRefits Integer indicating number of attempts to fit an observation
 #' if the first attempt fails
 #' @param verbose currently not used
-#' @param returnX Boolean. Return data with bdObj? Currently not implemented
 #' @param ... Secret
 #'
 #' @return Object of class 'bdotsObj', inherits from data.table
@@ -22,7 +21,7 @@
 #' @details This is step one of the three step bdots process. Things should be
 #' more or less straight forward. The only tricky part involves curveType. For now
 #' know that one can use doubleGauss(concave = TRUE/FALSE) or logistic(). Should
-#' be passed in as a function. See the vignette on customizing this
+#' be passed in as a call. See the vignette on customizing this
 #'
 #' @examples
 #' \dontrun{
@@ -52,7 +51,6 @@ bdotsFit <- function(data, # dataset
                      numRefits = 0,
                      cores = 0, # cores to use, 0 == 50% of available
                      verbose = FALSE,
-                     returnX = NULL,
                      ...) {
 
   if (cores < 1) cores <- detectCores()/2
@@ -62,8 +60,10 @@ bdotsFit <- function(data, # dataset
 
   ## Variable names on the dataset
   datVarNames <- c(y = y, subject = subject, time = time, group = group)
+  haveVars <- datVarNames %in% names(data)
   if (!all(datVarNames %in% names(data))) {
-    stop("need more specific error. Either subject, time, y, or group is not in dataset")
+    stopMsg <- paste0('"', datVarNames[!haveVars],'" is not a column of the dataset')
+    stop(stopMsg)
   }
 
   # for removing rho (need to adjust in case it's in (...))
@@ -71,35 +71,43 @@ bdotsFit <- function(data, # dataset
   if (!exists("rho")) {
     rho <- ifelse(cor, 0.9, 0)
   } else {
-    if(cor & (rho >= 1 | rho < 0)) {
+    if (cor & (rho >= 1 | rho < 0)) {
       warning("cor set to TRUE with invalid rho. Setting rho to 0.9")
       rho <- 0.9
     }
   }
 
-  ## Factors eff things up
+  ## Factors are bad, m'kay?
   dat <- setDT(data)
   dat[, (group) := lapply(.SD, as.character), .SDcols = group]
   dat[, (subject) := lapply(.SD, as.character), .SDcols = subject]
 
+  ## Let's only keep the columns we need (have not tested this yet)
+  ## Ok, let's try keeping everything in case we need correlation with fixed val
+  #dat <- dat[, c(y, time, subject, group), with = FALSE]
+
+
   timetest <- split(dat, by = group, drop = TRUE)
   timetest <- lapply(timetest, function(x) unique(x[[time]]))
   timeSame <- identical(Reduce(intersect, timetest, init = timetest[[1]]),
                         Reduce(union, timetest, init = timetest[[1]]))
-  if (!timeSame) stop("Yo, these times are different between groups, and until collin fixes it, that's going to make bdotsBoot wrong-ish")
-
-  ## This shuld work inside function. Let's check
-  if (any(dat[, .N, by = c(subject, time, group)]$N > 1)) {
-    warning("Some subjects have multiple observations for unique time. These will be averaged")
-    yval <- deparse(substitute(y))
-    dat[, substitute(y) := mean(get(y)), by = c(subject, time, group)]
-    dat <- unique(dat, by = c(subject, time, group))
+  #if (!timeSame) stop("Observed times are different between groups")
+  if (!timeSame) {
+    warning("Observed times are not identical between groups. This will result in weird plotting behavior")
   }
 
+  ## This should work inside function. Let's check
+  # if this happens, need to modify X for plots to work
+  # if (any(dat[, .N, by = c(subject, time, group)]$N > 1)) {
+  #   warning("Some subjects have multiple observations for unique time. These will be averaged")
+  #   yval <- deparse(substitute(y))
+  #   dat[, substitute(y) := mean(get(y)), by = c(subject, time, group)]
+  #   dat <- unique(dat, by = c(subject, time, group))
+  # }
+
   splitVars <- c(subject, group)
   newdat <- split(dat, by = splitVars, drop = TRUE)
 
-
   if (Sys.info()['sysname'] == "Darwin") {
     cl <- makePSOCKcluster(cores, setup_strategy = "sequential")
   } else {
@@ -108,6 +116,15 @@ bdotsFit <- function(data, # dataset
 
   invisible(clusterEvalQ(cl, {library(bdots)}))
 
+  ## Only for error checking, as devtools::load_all doesn't help
+  ## with parallel
+  # res <- vector("list", length(newdat))
+  # for (i in seq_along(newdat)) {
+  #   res[[i]] <- bdotsFitter(newdat[[i]], curveType, rho,
+  #                           numRefits, verbose = FALSE,
+  #                           splitVars = splitVars, datVarNames = datVarNames)
+  # }
+
   res <- parLapply(cl, newdat, bdotsFitter,
                    curveType = curveType,
                    rho = rho, numRefits = numRefits,
@@ -116,41 +133,51 @@ bdotsFit <- function(data, # dataset
                    datVarNames = datVarNames)
   stopCluster(cl)
 
+  ## Remove entries that had zero outcome variance
+  ## NOTE:: need to make option to remove all entries from subject
+  # to retain paired t-test
+  nullEntries <- vapply(res, is.null, logical(1))
+  if (sum(nullEntries) != 0) {
+    nn <- names(newdat)[which(nullEntries)]
+    message(paste0("Observations with zero outcome variance were removed (n = ", length(nn), "):"))
+    for (i in seq_along(nn)) {
+      message(nn[i])
+    }
+    res <- compact(res)
+  }
 
   ff <- attr(res[[1]], "formula")
   fitList <- rbindlist(res, fill = TRUE)
 
- ## If too large, should get "name" of data
- # and option to call it from global env
- if (is.null(returnX)) {
-   sz <- object.size(data)
-   if (sz < 1e8L) X <- data
- } else if (returnX) {
-   X <- data
- } else {
-   X <- NULL # for now
- }
+  ## If too large, should get "name" of data
+  ## and option to call it from global env
+  # if (is.null(returnX)) {
+  #   sz <- object.size(dat)
+  #   if (sz < 1e8L) X <- dat
+  # } else if (returnX) {
+  #   X <- dat
+  # } else {
+  #   X <- dat # for now
+  # }
 
- X_env <- new.env(parent = emptyenv())
- X_env$X <- X
+  X_env <- new.env(parent = emptyenv())
+  X_env$X <- dat
 
- ## Janky for now, but we want a groupName List
- # tmp <- unique(fitList[, group, with = FALSE])
- # tmp <- names(split(tmp, by = group))
- vals <- do.call(function(...) paste(..., sep = "."),
-                 unique(fitList[, group, with = FALSE]))
- groups <- list(groups = group,
-                vals = vals)
+  ## Janky for now, but we want a groupName List
+  vals <- do.call(function(...) paste(..., sep = "."),
+                  unique(fitList[, group, with = FALSE]))
+  groups <- list(groups = group,
+                 vals = vals)
 
- res <- structure(class = c("bdotsObj", "data.table", "data.frame"),
-                  .Data = fitList,
-                  formula = ff,
-                  curveType = curveName,
-                  call = match.call(),
-                  time = timetest[[1]],
-                  rho = rho,
-                  groups = groups,
-                  X = X_env)
+  res <- structure(class = c("bdotsObj", "data.table", "data.frame"),
+                   .Data = fitList,
+                   formula = ff,
+                   curveType = curveName,
+                   call = match.call(),
+                   time = timetest[[1]],
+                   rho = rho,
+                   groups = groups,
+                   X = X_env)
 }
 
 
